﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Gifitti.Models;
using System.Threading;
using Gifitti.Transformation;
using ImageMagick;

namespace Gifitti.View_Models
{
    /// <summary>
    /// System Controller, binded to MainForm View
    /// This File abstracts all control elements away from the view
    ///     elements stored in the remainder of the partial class
    /// </summary>
    public partial class MainForm : Form
    {
        #region Fields
        GifModel gm;
        HelpContext f2 = new HelpContext();
        // GifImage _currentGif; <- used to encapsulate info later 
        private const int widthBuffer = 20;
        private const int heightBuffer = 60;
        Image globalGif;
        #endregion
        #region Constructors
        /// <summary>
        /// Returns a new MainForm instance, and initializes the view.
        /// </summary>
        public MainForm()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {

        }
        #endregion
        #region Buttons
        
        #endregion
        #region SystemChecks
        /// <summary>
        /// Validates that a GIF is loaded and in the model instance. 
        /// If not instance is loaded a prompt is displayed to the user.
        /// </summary>
        /// <returns> True iff a gif file is loaded and stored in the model. False else. </returns>
        private Boolean chkImage()
        {
            try
            {
                int Length = globalGif.GetFrameCount(FrameDimension.Time);
                return true;
            }
            catch (Exception)
            {
                MessageBox.Show("Must load a gif before this option is availiable!");
            }
            return false;
        }
        #endregion
        #region MenuItems

        /// <summary>
        /// On event invocation the loaded gif is decomposed into its frames and saved to a specified folder as JPEG images.
        /// </summary>
        /// <param name="sender"> Caller of the event. </param>
        /// <param name="e"> Event arguments associated with this event. </param>
        private void MenuItemExportJPEG(object sender, EventArgs e)
        {
            exportFrames(ImageFormat.Jpeg);  
        }

        /// <summary>
        /// On event invocation the loaded gif is decomposed into its frames and saved to a specified folder as PNG images.
        /// </summary>
        /// <param name="sender"> Caller of the event. </param>
        /// <param name="e"> Event arguments associated with this event. </param>
        private void MenuItemExportPNG(object sender, EventArgs e)
        {
            exportFrames(ImageFormat.Png);
        }

        private static Dictionary<ImageFormat, string> IMAGE_TO_EXTENSION = new Dictionary<ImageFormat, string>()
        {
            {ImageFormat.Bmp,   "bmp"   },
            {ImageFormat.Tiff,  "tiff"  },
            {ImageFormat.Jpeg,  "jpeg"  },
            {ImageFormat.Png,   "png"   }
        };

        /// <summary>
        /// On event invocation the loaded gif is decomposed into its frames and saved to a specified folder as BMP (Bitmap) images.
        /// </summary>
        /// <param name="sender"> Caller of the event. </param>
        /// <param name="e"> Event arguments associated with this event. </param>
        private void MenuItemExportBMP(object sender, EventArgs e)
        {
            exportFrames(ImageFormat.Bmp);
        }

    
        public void exportFrames(ImageFormat format)
        {
            if (!chkImage())
                return;

            String path = "";
            if (folderBrowserDialog.ShowDialog() == DialogResult.OK)
                path = folderBrowserDialog.SelectedPath;


            for (int i = gm.startFrame; i < gm.endFrame; i++)
            {
                var a = gm.GetFrame(i);
                a.Save(path + @"\frame" + i + @"." + IMAGE_TO_EXTENSION[format], format);
            }
        }

        /// <summary>
        /// On event invocation the loaded gif is decomposed into its frames and saved to a specified folder as TIFF images.
        /// </summary>
        /// <param name="sender"> Caller of the event. </param>
        /// <param name="e"> Event arguments associated with this event. </param>
        private void MenuItemExportTIFF(object sender, EventArgs e)
        {
            exportFrames(ImageFormat.Tiff);
        }

        /// <summary>
        /// Controls the start stop behaviour. Execution of this command
        /// toggles if the GIF is playing or paused.
        /// </summary>
        /// <param name="sender"> Caller of the event. </param>
        /// <param name="e"> Event arguments associated with this event. </param>
        private void StartStopClickEvent(object sender, EventArgs e)
        {
            if (timer1.Enabled == true)
            {
                button3.Text = "Start";
                gifView.Enabled = false;
                timer1.Enabled = false;
            }
            else
            {
                button3.Text = "Stop";
                gifView.Enabled = true;
                timer1.Enabled = true;
            }
        }

        /// <summary>
        /// GIF playback controller
        /// </summary>
        /// <param name="sender"> Caller of the event. </param>
        /// <param name="e"> Event arguments associated with this event. </param>
        private void AdvanceFrameTimer(object sender, EventArgs e)
        {
            //get next frame
            Thread.Sleep(gm.delay);
            gifView.Image = gm.GetNextFrame();
        }

        /// <summary>
        /// Static link between the start frame text box and model start frame.
        /// </summary>
        /// <param name="sender"> Caller of the event. </param>
        /// <param name="e"> Event arguments associated with this event. </param>
        private void StartFrameTextChanged(object sender, EventArgs e)
        {
            int start = 0;
            bool pass = int.TryParse(textBox1.Text, out start);
            if (pass && start >= 0 && start < gm.numberOfFrames)
            {
                gm.startFrame = start;
            }
        }

        /// <summary>
        /// Static link between the stop frame text box and model stop frame.
        /// </summary>
        /// <param name="sender"> Caller of the event. </param>
        /// <param name="e"> Event arguments associated with this event. </param>
        private void StopFrameTextChanged(object sender, EventArgs e)
        {
            int end = 0;
            bool pass = int.TryParse(textBox2.Text, out end);
            if (pass & end >= 0)
            {
                gm.endFrame = end;
            }

        }

        /// <summary>
        /// Event controlling the Save File Dialog.
        /// </summary>
        /// <param name="sender"> Caller of the event. </param>
        /// <param name="e"> Event arguments associated with this event. </param>
        private void MenuItemFileSaveAs(object sender, EventArgs e)
        {
            //TransformGif.saveThePlayback(gm, delay);
            if (!chkImage())
                return;
            String path = null;
            saveGifFileDialog.Filter = "GIF Files|*.gif";
            if (saveGifFileDialog.ShowDialog() == DialogResult.OK)
                path = saveGifFileDialog.FileName;
            //Handel Close
            if (path == null)
                return;
            //TODO temp write
            Console.WriteLine(path);
            try {
                gm.saveGif(path);
            } catch (DirectoryNotFoundException)
            {
                MessageBox.Show("Save Path Invalid!");
            }
            
        }

        #endregion
        #region File Dialogs
        /// <summary>
        /// Statically binded function executed when openGifDialog has its button "Ok" clicked.
        /// </summary>
        /// <param name="sender"> Caller of the event. </param>
        /// <param name="e"> Event arguments associated with this event. </param>
        private void openGifDialog_FileOk(object sender, CancelEventArgs e)
        {

        }
        #endregion

        /// <summary>
        /// Launches a file dialog to load a GIF. Users can only select valid files.
        /// </summary>
        /// <param name="sender"> Caller of the event. </param>
        /// <param name="e"> Event arguments associated with this event. </param>
        private void MenuOpenFile(object sender, EventArgs e)
        {
            openGifFileDialog.Filter = "GIF Files|*.gif";
            DialogResult result = openGifFileDialog.ShowDialog(); // Show the dialog.
            if (result == DialogResult.OK) // Test result.
            {
                string file = openGifFileDialog.FileName;
                try
                {
                    try
                    {
                        if (!Path.GetExtension(file).ToLower().Equals(".gif"))
                            throw new FileLoadException("Invalid Extension Specified");
                        gm?.Dispose();
                        gm = new GifModel(file);
                        gm.ReverseAtEnd = false;
                        Image loadedGif = gm.gifImage;
                        globalGif = loadedGif;
                        //gifView.Image = loadedGif;

                        gifView.Width = loadedGif.Width;
                        gifView.Height = loadedGif.Height;
                        ClientSize = new Size(loadedGif.Width + widthBuffer, loadedGif.Height + heightBuffer);
                    }
                    catch (FileLoadException)
                    {
                        MessageBox.Show("Specify a GIF file.");
                        return;
                    }
                }

                catch (IOException)
                {
                }

                CenterToScreen();

                //Focus the gif:
                gifView.Select();
                gifView.Focus();
                ActiveControl = gifView;

                //Start Animating   
                timer1.Enabled = true;

                //Enable frame selection
                textBox1.Enabled = true;
                textBox2.Enabled = true;
                button3.Enabled = true;

                //set the initial frames in textbox
                textBox1.Text = "0";
                textBox2.Text = gm.numberOfFrames.ToString();
            }

        }
        /// <summary>
        /// Loads the Help Menu Form.
        /// </summary>
        /// <param name="sender"> Caller of the event. </param>
        /// <param name="e"> Event arguments associated with this event. </param>
        private void menuOpenHelp(object sender, EventArgs e)
        {
            if(f2.IsDisposed)
                f2 = new HelpContext();
            if (!f2.Visible)
                f2.Show();
        }

        /// <summary>
        /// Static Binding to the GIF Frame rate slider, slecting current frame rate.
        /// Delay ranges from 0-100ms
        /// </summary>
        /// <param name="sender"> Caller of the event. </param>
        /// <param name="e"> Event arguments associated with this event. </param>
        private void GifFPSSliderUpdate(object sender, EventArgs e)
        {
            if (gm != null)
                gm.delay = (11-trackBar1.Value)*gm.originalDelay;
        }

        private void MenuItemResize(object sender, EventArgs e)
        {
            if (chkImage()) {
                int height = gifView.Height;
                int width = gifView.Width;
                SaveAsXbyYPrompt save = new SaveAsXbyYPrompt(width,height,gm);
                save.Show();
            }
        }

        private void trackBar1_Scroll(object sender, EventArgs e)
        {

        }

        
    }
}
